-------------------------------------------------------------------------------
--|> Frames
-- Creates and updates the addon frames and widgets
-------------------------------------------------------------------------------
--|> UPVALUE GLOBALS
-------------------------------------------------------------------------------
local weizPVP = weizPVP
local InCombatLockdown = InCombatLockdown

-------------------------------------------------------------------------------
--|> LIBS
-------------------------------------------------------------------------------
local SM = LibStub:GetLibrary("LibSharedMedia-3.0")

-------------------------------------------------------------------------------
--|> LOCAL FUNCTIONS
-------------------------------------------------------------------------------

--=> Adjust Scroll Offset
-----------------------------------------------------------------------
function weizPVP.AdustScrollOffset(self, value)
    if not InCombatLockdown() then
        local offsetMax = weizPVP.ListFrame:GetHeight() - weizPVP.ScrollFrame:GetHeight()
        local offset
        if value > 0 then
            offset = weizPVP.ScrollFrame:GetVerticalScroll() - (weizPVP.Options.Bars.RowHeight + weizPVP.Options.Bars.VerticalSpacing)
        else
            offset = weizPVP.ScrollFrame:GetVerticalScroll() + (weizPVP.Options.Bars.RowHeight + weizPVP.Options.Bars.VerticalSpacing)
        end
        if offset < 0 then
            weizPVP.ScrollFrame:SetVerticalScroll(0)
        elseif offset > offsetMax then
            weizPVP.ScrollFrame:SetVerticalScroll(offsetMax)
        else
            weizPVP.ScrollFrame:SetVerticalScroll(offset)
        end
    end
end

--=> AlignFrameToPixels
-----------------------------------------------------------------------
function weizPVP:AlignFrameToPixels(frame)
    if not InCombatLockdown() then
        local x, y = frame:GetCenter()
        if x and y then
            x, y = x * frame:GetEffectiveScale(), y * frame:GetEffectiveScale()
            local fScale = frame:GetScale()
            x, y = x / fScale, y / fScale
            frame:SetPoint("CENTER", nil, "BOTTOMLEFT", weizPVP.ScaleCoords(x, y))
        end
    end
end

-------------------------------------------------------------------------------
--|> CONFIGURE HEADER
-------------------------------------------------------------------------------
local HeaderFrameHighlight

function weizPVP:CreateCoreFrames()
    --=> weizPVP.CoreFrame: SETTINGS
    -----------------------------------------------------------------------
    weizPVP.CoreFrame = CreateFrame("Frame", "weizPVP.CoreFrame", UIParent)
    weizPVP.CoreFrame:SetClampedToScreen(true)
    weizPVP.CoreFrame:SetClipsChildren(true)
    weizPVP.CoreFrame:SetFrameStrata("HIGH")
    weizPVP.CoreFrame:SetFrameLevel(1)
    weizPVP.CoreFrame:SetPoint("CENTER", UIParent)
    weizPVP.CoreFrame:SetSize(weizPVP.Options.Frames.Width, (weizPVP.Options.Frames.Height - weizPVP.Options.Bars.VerticalSpacing))
    weizPVP.CoreFrame:SetMovable(true)
    weizPVP.CoreFrame:EnableMouse(true)
    weizPVP.CoreFrame:SetDontSavePosition(true)
    weizPVP.CoreFrame:SetBackdrop(
        {
            bgFile = SM:Fetch("background", weizPVP.Options.Frames.BackgroundTexture),
            tile = false
        }
    )
    weizPVP.CoreFrame:SetBackdropColor(unpack(weizPVP.Options.Frames.BackgroundColor))
    weizPVP.CoreFrame:SetResizable(true)
    weizPVP.CoreFrame:SetMinResize(180, weizPVP.Options.Bars.RowHeight)
    weizPVP.CoreFrame:SetMaxResize(500, weizPVP.Options.Frames.List.Height)
    weizPVP.CoreFrame:Show()
    weizPVP.CoreFrame.isLocked = false
    weizPVP.CoreFrame.isPinned = false

    --=> weizPVP.ScrollFrame: SETTINGS
    --> The frame that is a child of weizPVP.CoreFrame that will be scolling with content
    -----------------------------------------------------------------------
    weizPVP.ScrollFrame = CreateFrame("ScrollFrame", "weizPVP.ScrollFrame", weizPVP.CoreFrame, "weizPVPSecureScrollFrame")
    weizPVP.ScrollFrame:EnableMouseWheel(true)
    weizPVP.ScrollFrame:SetClipsChildren(true)
    weizPVP.ScrollFrame:SetPoint("TOPLEFT", weizPVP.CoreFrame, "TOPLEFT", weizPVP.Options.Frames.BorderSize, 0)
    weizPVP.ScrollFrame:SetPoint("BOTTOMRIGHT", weizPVP.CoreFrame, "BOTTOMRIGHT", (-1 * weizPVP.Options.Frames.BorderSize), 0)

    --> MOUSEWHEEL
    weizPVP.ScrollFrame:SetScript(
        "OnMouseWheel",
        function(self, value)
            weizPVP.AdustScrollOffset(self, value)
        end
    )

    --=> weizPVP.ListFrame: SETTINGS
    --> The frame that is a child of weizPVP.CoreFrame that will be scolling with content
    -----------------------------------------------------------------------
    weizPVP.ListFrame = CreateFrame("Frame", "weizPVP.ListFrame", weizPVP.ScrollFrame)
    weizPVP.ListFrame:SetPoint("CENTER", weizPVP.ScrollFrame, "CENTER", 0, 1)
    weizPVP.ListFrame:SetPoint("TOP", weizPVP.CoreFrame, "BOTTOM", 0, 1)
    weizPVP.ListFrame:SetHeight(weizPVP.Options.Frames.List.Height)
    weizPVP.ListFrame:SetWidth(weizPVP.ScrollFrame:GetWidth() - weizPVP.Options.Frames.BorderSize)
    weizPVP.ScrollFrame:SetScrollChild(weizPVP.ListFrame)

    --=> weizPVP.HeaderFrame: SETTINGS
    -----------------------------------------------------------------------
    weizPVP.HeaderFrame = CreateFrame("Frame", "weizPVP.HeaderFrame", UIParent)
    weizPVP.HeaderFrame:SetClampedToScreen(true)
    weizPVP.HeaderFrame:SetPoint("BOTTOMLEFT", weizPVP.CoreFrame, "TOPLEFT")
    weizPVP.HeaderFrame:SetPoint("BOTTOMRIGHT", weizPVP.CoreFrame, "TOPRIGHT")
    weizPVP.HeaderFrame:SetHeight(weizPVP.Options.Frames.Header.Height)
    weizPVP.HeaderFrame:SetDontSavePosition(true)
    weizPVP.HeaderFrame:SetFrameStrata("HIGH")
    weizPVP.HeaderFrame:EnableMouse(true)
    weizPVP.HeaderFrame:SetBackdrop(
        {
            bgFile = SM:Fetch("background", "weizPAK: Background"),
            tile = false,
            edgeFile = SM:Fetch("border", "weizPAK: Border"),
            edgeSize = 1,
            insets = {left = 0, right = 0, top = 0, bottom = 0}
        }
    )
    weizPVP.HeaderFrame:SetBackdropColor(0.01, 0.01, 0.01, 0.9)
    weizPVP.HeaderFrame:SetBackdropBorderColor(0, 0, 0, 1)

    --=> weizPVP.HeaderFrame: Scripts
    -----------------------------------------------------------------------

    --> ON MOUSE DOWN
    -- * Move core frame if window allows (unlocked/unpinned/not-clamped)
    -------------------------------------------
    weizPVP.HeaderFrame:SetScript(
        "OnMouseDown",
        function(self, button)
            if (((not weizPVP.CoreFrame.isLocked) or (weizPVP.CoreFrame.isLocked == false)) and (button == "LeftButton") and not weizPVP.CoreFrame.isPinned) then
                weizPVP.CoreFrame:StartMoving()
                weizPVP.CoreFrame:SetClampRectInsets(0, 0, weizPVP.Options.Frames.Header.Height + 1, -1 * (weizPVP.Options.Frames.Footer.Height))
                weizPVP.CoreFrame.isMoving = true
            elseif button == "RightButton" then
                weizPVP:TogglePinned()
            end
        end
    )

    --> ON MOUSE UP
    -- * Stop frame
    -------------------------------------------
    weizPVP.HeaderFrame:SetScript(
        "OnMouseUp",
        function(self, button)
            if (weizPVP.CoreFrame.isMoving) then
                weizPVP.CoreFrame:StopMovingOrSizing()
                weizPVP.CoreFrame.isMoving = false
                weizPVP:SaveCoreFramePosition()
                if button == "RightButton" then
                end
            end
        end
    )
    --> ON MOUSE ENTER
    -------------------------------------------
    weizPVP.HeaderFrame:SetScript(
        "OnEnter",
        function(self, button)
            UIFrameFadeIn(HeaderFrameHighlight, 0.3, 0, 1)
        end
    )
    --> ON MOUSE LEAVE
    -------------------------------------------
    weizPVP.HeaderFrame:SetScript(
        "OnLeave",
        function(self, button)
            UIFrameFadeOut(HeaderFrameHighlight, 0.2, 1, 0)
        end
    )

    --=> weizPVP.OuterIcons
    --> Frame that contains icons outside of the core frame
    -----------------------------------------------------------------------
    weizPVP.OuterIconsFrame = CreateFrame("Frame", "weizPVP.OuterIconsFrame", weizPVP.HeaderFrame)
    weizPVP.OuterIconsFrame:SetPoint("TOPLEFT", weizPVP.ScrollFrame, "TOPLEFT", -24, 0)
    weizPVP.OuterIconsFrame:SetPoint("BOTTOMRIGHT", weizPVP.ScrollFrame, "BOTTOMRIGHT", 0, 0)
    weizPVP.OuterIconsFrame:SetClipsChildren(true)

    --=> Header Frame Highlight
    -----------------------------------------------------------------------
    HeaderFrameHighlight = CreateFrame("Frame", "HeaderFrameHighlight", weizPVP.HeaderFrame)
    HeaderFrameHighlight:SetAllPoints(true)
    HeaderFrameHighlight.Texture = HeaderFrameHighlight:CreateTexture(nil, "BACKGROUND")
    HeaderFrameHighlight.Texture:SetAllPoints(true)
    HeaderFrameHighlight.Texture:SetTexture("Interface\\AddOns\\weizPVP\\Textures\\white16x16.tga")
    HeaderFrameHighlight.Texture:SetVertexColor(0.6, 0.6, 0.6, 1)
    HeaderFrameHighlight.Texture:SetBlendMode("BLEND")
    HeaderFrameHighlight:SetFrameLevel(1)
    HeaderFrameHighlight:Hide()
    weizPVP.HeaderFrameHighlight = HeaderFrameHighlight

    --=> weizPVP.HeaderFrame.Text: SETTINGS
    -----------------------------------------------------------------------
    weizPVP.HeaderFrame.Text = weizPVP.HeaderFrame:CreateFontString(nil, "OVERLAY", nil)
    weizPVP.HeaderFrame.Text:SetPoint("LEFT", weizPVP.HeaderFrame, "LEFT", 8, 0)
    weizPVP.HeaderFrame.Text:SetFont(SM:Fetch("font", weizPVP.Options.Frames.Header.Font), weizPVP.Options.Frames.Header.FontSize, weizPVP.Options.Frames.Header.FontOutline)
    weizPVP.HeaderFrame.Text:SetJustifyH("LEFT")
    weizPVP.HeaderFrame.Text:SetJustifyV("MIDDLE")
    weizPVP.HeaderFrame.Text:SetTextColor(1, 1, 1, 1)
    weizPVP.HeaderFrame.Text:SetText("NEARBY |cff777777:|r |cFF26FF540|r")
    weizPVP.HeaderFrame.Text:SetScale(1)

    --=> weizPVP. Create Header Icons
    -----------------------------------------------------------------------
    weizPVP:CreateHeaderIcons()

    --=> weizPVP.FooterFrame: SETTINGS
    -----------------------------------------------------------------------
    weizPVP.FooterFrame = CreateFrame("Frame", "weizPVP.FooterFrame", weizPVP.HeaderFrame)
    weizPVP.FooterFrame:SetClampedToScreen(true)
    weizPVP.FooterFrame:SetPoint("TOPLEFT", weizPVP.CoreFrame, "BOTTOMLEFT")
    weizPVP.FooterFrame:SetPoint("TOPRIGHT", weizPVP.CoreFrame, "BOTTOMRIGHT")
    weizPVP.FooterFrame:EnableMouse(false)
    weizPVP.FooterFrame:SetHeight(weizPVP.Options.Frames.Footer.Height)
    weizPVP.FooterFrame:SetBackdrop(
        {
            bgFile = SM:Fetch("background", "weizPAK: Background"),
            tile = false,
            edgeFile = SM:Fetch("border", "weizPAK: Border"),
            edgeSize = 1,
            insets = {left = 0, right = 0, top = 0, bottom = 0}
        }
    )
    weizPVP.FooterFrame:SetBackdropColor(0.01, 0.01, 0.01, 0.8)
    weizPVP.FooterFrame:SetBackdropBorderColor(0, 0, 0, 1)
    weizPVP.FooterFrame:SetScale(1)

    --=> weizPVP.FooterFrame.Text: SETTINGS
    -----------------------------------------------------------------------
    weizPVP.FooterFrame.Text = weizPVP.FooterFrame:CreateFontString(nil, "OVERLAY", nil)
    weizPVP.FooterFrame.Text:SetPoint("TOPLEFT", weizPVP.FooterFrame, "TOPLEFT", 2, 0)
    weizPVP.FooterFrame.Text:SetPoint("BOTTOMRIGHT", weizPVP.FooterFrame, "BOTTOMRIGHT")
    weizPVP.FooterFrame.Text:SetFont(SM:Fetch("font", weizPVP.Options.Frames.Footer.Font), weizPVP.Options.Frames.Footer.FontSize, weizPVP.Options.Frames.Footer.FontOutline)
    weizPVP.FooterFrame.Text:SetJustifyH("LEFT")
    weizPVP.FooterFrame.Text:SetJustifyV("MIDDLE")
    weizPVP.FooterFrame.Text:SetText("")

    weizPVP:CreateResizeGrip()
end

function weizPVP:SaveCoreFramePosition()
    weizPVP.Options.Frames.X = weizPVP.CoreFrame:GetLeft()
    weizPVP.Options.Frames.Y = weizPVP.CoreFrame:GetTop()
    weizPVP.Options.Frames.Width = weizPVP.FooterFrame:GetWidth()
    weizPVP.Options.Frames.Height = weizPVP.CoreFrame:GetHeight()
    weizPVP:SaveAddonData()
end

function weizPVP:SetCoreFramePosition(x, y, width, height)
    weizPVP.CoreFrame:SetPoint("TOPLEFT", UIParent, "BOTTOMLEFT", x, y)
    weizPVP.HeaderFrame:ClearAllPoints()
    weizPVP.HeaderFrame:SetPoint("BOTTOMLEFT", weizPVP.CoreFrame, "TOPLEFT")
    weizPVP.HeaderFrame:SetPoint("BOTTOMRIGHT", weizPVP.CoreFrame, "TOPRIGHT")
    weizPVP.CoreFrame:SetWidth(width)
    weizPVP.CoreFrame:SetHeight(height)
    weizPVP:SaveCoreFramePosition()
end

function weizPVP:TogglePinned(value)
    if weizPVP.Options.WindowLocked == true then
        weizPVP:PrintAddonMessage("Window is currently |cffff3838LOCKED|r. Unlock the window before using the pin option.")
        return
    end

    if value ~= nil then
        --> non nil value; apply value
        if value == true then
            weizPVP.CoreFrame:SetMovable(false)
            weizPVP.HeaderFrame:SetMovable(false)
            weizPVP.HeaderFrame:EnableMouse(true)
            weizPVP.CoreFrame.isLocked = false
            weizPVP.CoreFrame.isPinned = true
            weizPVP:PrintAddonMessage("Window is now |cff42dcf4PINNED|r.")
            weizPVP.Options.WindowPinned = true
        else
            weizPVP.CoreFrame:SetMovable(true)
            weizPVP.HeaderFrame:SetMovable(true)
            weizPVP.HeaderFrame:EnableMouse(true)
            weizPVP.CoreFrame.isLocked = false
            weizPVP.CoreFrame.isPinned = false
            weizPVP:PrintAddonMessage("Window is now |cff42dcf4UNPINNED|r.")
            weizPVP.Options.WindowPinned = false
        end
    else
        --> nil value; toggle pinning
        if not weizPVP.Options.WindowPinned then
            --> unpin frame
            weizPVP.CoreFrame:SetMovable(false)
            weizPVP.HeaderFrame:SetMovable(false)
            weizPVP.HeaderFrame:EnableMouse(true)
            weizPVP.CoreFrame.isLocked = false
            weizPVP.CoreFrame.isPinned = true
            weizPVP:PrintAddonMessage("Window is now |cff42dcf4PINNED|r.")
            weizPVP.Options.WindowPinned = true
        else
            --> pin frame
            weizPVP.CoreFrame:SetMovable(true)
            weizPVP.HeaderFrame:SetMovable(true)
            weizPVP.HeaderFrame:EnableMouse(true)
            weizPVP.CoreFrame.isLocked = false
            weizPVP.CoreFrame.isPinned = false
            weizPVP:PrintAddonMessage("Window is now |cff42dcf4UNPINNED|r.")
            weizPVP.Options.WindowPinned = false
        end
    end
    weizPVP:SaveAddonData()
    return
end

function weizPVP:ToggleLocked(value)
    if value ~= nil and weizPVP.Options.WindowLocked ~= value then --> value given and is different; apply value
        if value == true then
            weizPVP.CoreFrame:SetMovable(false)
            weizPVP.HeaderFrame:SetMovable(false)
            weizPVP.CoreFrame.isLocked = true
            weizPVP:PrintAddonMessage("Window is now |cffff3838LOCKED|r")
        else
            weizPVP.CoreFrame:SetMovable(true)
            weizPVP.HeaderFrame:SetMovable(true)
            weizPVP.CoreFrame.isLocked = false
            weizPVP:PrintAddonMessage("Window is now |cff37ff37UNLOCKED|r.")
        end

        weizPVP.Options.WindowLocked = value
    else --> no value given; toggle lock
        if weizPVP.Options.WindowLocked == true then
            weizPVP.CoreFrame:SetMovable(true)
            weizPVP.HeaderFrame:SetMovable(true)
            weizPVP.CoreFrame.isLocked = false
            weizPVP:PrintAddonMessage("Window is now |cff37ff37UNLOCKED|r.")
        else
            weizPVP.CoreFrame:SetMovable(false)
            weizPVP.HeaderFrame:SetMovable(false)
            weizPVP.CoreFrame.isLocked = true
            weizPVP:PrintAddonMessage("Window is now |cffff3838LOCKED|r")
        end
    end
    --> Save Options
    weizPVP:SaveAddonData()
    return
end
